@@ -101,13 +101,28 @@ class ProbabilityEstimator(object):
         )
 
 
-def _lang_dict_paths():
+def _iter_entry_points(*args, **kwargs):
+    """ Like pkg_resources.iter_entry_points, but uses a WorkingSet which
+    is not populated at startup. This ensures that all entry points
+    are picked up, even if a package which provides them is installed
+    after the current process is started.
+
+    The main use case is to make ``!pip install pymorphy2`` work
+    within a Jupyter or Google Colab notebook.
+    See https://github.com/kmike/pymorphy2/issues/131
+    """
     import pkg_resources
+    ws = pkg_resources.WorkingSet()
+    return ws.iter_entry_points(*args, **kwargs)
+
+
+def _lang_dict_paths():
     paths = dict(
         (pkg.name, pkg.load().get_path())
-        for pkg in pkg_resources.iter_entry_points('pymorphy2_dicts')
+        for pkg in _iter_entry_points('pymorphy2_dicts')
     )
 
+    # discovery of pymorphy2 v0.8 dicts
     try:
         import pymorphy2_dicts
         paths['ru-old'] = pymorphy2_dicts.get_path()
