@@ -9,6 +9,10 @@ not fully compatible with OpenCorpora.
 from __future__ import absolute_import, unicode_literals
 import logging
 import collections
+
+from pymorphy2.utils import with_progress
+from pymorphy2.cache import memoized_with_single_argument
+
 logger = logging.getLogger(__name__)
 
 
@@ -26,7 +30,7 @@ def simplify_tags(parsed_dict, skip_space_ambiguity=True):
     logger.debug("%d duplicate tags will be removed", len(tag_replaces))
 
     logger.info("simplifying tags: fixing")
-    for lex_id in parsed_dict.lexemes:
+    for lex_id in with_progress(parsed_dict.lexemes, "Simplifying tags"):
         new_lexeme = [
             (word, _simplify_tag(tag, tag_replaces))
             for word, tag in parsed_dict.lexemes[lex_id]
@@ -34,11 +38,28 @@ def simplify_tags(parsed_dict, skip_space_ambiguity=True):
         parsed_dict.lexemes[lex_id] = new_lexeme
 
 
+def drop_unsupported_parses(parsed_dict):
+    """
+    Remove unsupported parses from OpenCorpora dictionary.
+
+    In particular, lexemes with Init tags are removed
+    because pymorphy2 handles them differently.
+    """
+    logger.info("dropping unsupported parses")
+    for lex_id in parsed_dict.lexemes:
+        parsed_dict.lexemes[lex_id] = [
+            (word, tag) for word, tag in parsed_dict.lexemes[lex_id]
+            if 'Init' not in tag
+        ]
+
+
+@memoized_with_single_argument({})
 def tag2grammemes(tag_str):
     """ Given tag string, return tag grammemes """
     return _split_grammemes(replace_redundant_grammemes(tag_str))
 
 
+@memoized_with_single_argument({})
 def replace_redundant_grammemes(tag_str):
     """ Replace 'loc1', 'gen1' and 'acc1' grammemes in ``tag_str`` """
     return tag_str.replace('loc1', 'loct').replace('gen1', 'gent').replace('acc1', 'accs')
@@ -103,6 +124,6 @@ def _simplify_tag(tag, tag_replaces):
 
 
 def _itertags(parsed_dict):
-    for lex_id in parsed_dict.lexemes:
+    for lex_id in with_progress(parsed_dict.lexemes, "Looking for tag spellings"):
         for word, tag in parsed_dict.lexemes[lex_id]:
             yield tag
