@@ -20,7 +20,6 @@ except AttributeError:
 import pymorphy2
 from pymorphy2 import tagset
 from pymorphy2 import dawg
-from pymorphy2.constants import PARADIGM_PREFIXES, PREDICTION_PREFIXES
 from pymorphy2.utils import json_write, json_read
 
 logger = logging.getLogger(__name__)
@@ -28,10 +27,15 @@ logger = logging.getLogger(__name__)
 CURRENT_FORMAT_VERSION = '2.4'
 
 LoadedDictionary = collections.namedtuple('LoadedDictionary', [
-    'meta', 'gramtab', 'suffixes', 'paradigms', 'words',
-    'prediction_prefixes', 'prediction_suffixes_dawgs',
-    'Tag', 'paradigm_prefixes']
-)
+    'meta',
+    'gramtab',
+    'suffixes',
+    'paradigms',
+    'words',
+    'prediction_suffixes_dawgs',
+    'Tag',
+    'paradigm_prefixes',
+])
 
 
 def load_dict(path, gramtab_format='opencorpora-int'):
@@ -42,7 +46,7 @@ def load_dict(path, gramtab_format='opencorpora-int'):
 
     _f = lambda p: os.path.join(path, p)
 
-    meta = _load_meta(_f('meta.json'))
+    meta = load_meta(_f('meta.json'))
     _assert_format_is_compatible(meta, path)
 
     Tag = _load_tag_class(gramtab_format, _f('grammemes.json'))
@@ -51,11 +55,14 @@ def load_dict(path, gramtab_format='opencorpora-int'):
     gramtab = [Tag(tag_str) for tag_str in str_gramtab]
 
     suffixes = json_read(_f('suffixes.json'))
-    paradigm_prefixes = json_read(_f('paradigm-prefixes.json'))
     paradigms = _load_paradigms(_f('paradigms.array'))
     words = dawg.WordsDawg().load(_f('words.dawg'))
 
-    prediction_prefixes = dawg.DAWG().load(_f('prediction-prefixes.dawg'))
+    try:
+        paradigm_prefixes = meta["compile_options"]["paradigm_prefixes"]
+    except KeyError:
+        # support dicts v2.4
+        paradigm_prefixes = json_read(_f('paradigm-prefixes.json'))
 
     prediction_suffixes_dawgs = []
     for prefix_id in range(len(paradigm_prefixes)):
@@ -63,12 +70,19 @@ def load_dict(path, gramtab_format='opencorpora-int'):
         assert os.path.exists(fn)
         prediction_suffixes_dawgs.append(dawg.PredictionSuffixesDAWG().load(fn))
 
-    return LoadedDictionary(meta, gramtab, suffixes, paradigms, words,
-                            prediction_prefixes, prediction_suffixes_dawgs,
-                            Tag, paradigm_prefixes)
+    return LoadedDictionary(
+        meta=meta,
+        gramtab=gramtab,
+        suffixes=suffixes,
+        paradigms=paradigms,
+        words=words,
+        prediction_suffixes_dawgs=prediction_suffixes_dawgs,
+        Tag=Tag,
+        paradigm_prefixes=paradigm_prefixes,
+    )
 
 
-def save_compiled_dict(compiled_dict, out_path):
+def save_compiled_dict(compiled_dict, out_path, source_name, language_code):
     """
     Save a compiled_dict to ``out_path``
     ``out_path`` should be a name of folder where to put dictionaries.
@@ -100,10 +114,6 @@ def save_compiled_dict(compiled_dict, out_path):
     for prefix_id, prediction_suffixes_dawg in enumerate(compiled_dict.prediction_suffixes_dawgs):
         prediction_suffixes_dawg.save(_f('prediction-suffixes-%s.dawg' % prefix_id))
 
-
-    dawg.DAWG(PREDICTION_PREFIXES).save(_f('prediction-prefixes.dawg'))
-    json_write(_f('paradigm-prefixes.json'), PARADIGM_PREFIXES)
-
     logger.debug("computing metadata..")
 
     def _dawg_len(dawg):
@@ -117,12 +127,13 @@ def save_compiled_dict(compiled_dict, out_path):
     for prediction_suffixes_dawg in compiled_dict.prediction_suffixes_dawgs:
         prediction_suffixes_dawg_lenghts.append(_dawg_len(prediction_suffixes_dawg))
 
-    meta = [
+    write_meta(_f('meta.json'), [
+        ['language_code', language_code],
         ['format_version', CURRENT_FORMAT_VERSION],
         ['pymorphy2_version', pymorphy2.__version__],
         ['compiled_at', datetime.datetime.utcnow().isoformat()],
 
-        ['source', 'opencorpora.org'],
+        ['source', source_name],
         ['source_version', compiled_dict.parsed_dict.version],
         ['source_revision', compiled_dict.parsed_dict.revision],
         ['source_lexemes_count', len(compiled_dict.parsed_dict.lexemes)],
@@ -134,16 +145,12 @@ def save_compiled_dict(compiled_dict, out_path):
         ['suffixes_length', len(compiled_dict.suffixes)],
 
         ['words_dawg_length', words_dawg_len],
-        ['prediction_options', compiled_dict.prediction_options],
+        ['compile_options', compiled_dict.compile_options],
         ['prediction_suffixes_dawg_lengths', prediction_suffixes_dawg_lenghts],
-        ['prediction_prefixes_dawg_length', len(PREDICTION_PREFIXES)],
-        ['paradigm_prefixes_length', len(PARADIGM_PREFIXES)],
-    ]
-
-    json_write(_f('meta.json'), meta, indent=4)
+    ])
 
 
-def _load_meta(filename):
+def load_meta(filename):
     """ Load metadata. """
     meta = json_read(filename, parse_float=str)
     if hasattr(collections, 'OrderedDict'):
@@ -151,6 +158,20 @@ def _load_meta(filename):
     return dict(meta)
 
 
+def write_meta(filename, meta):
+    """ Save metadata to a file. """
+    if isinstance(meta, dict):
+        meta = list(meta.items())
+    json_write(filename, meta)
+
+
+def update_meta(filename, extra):
+    """ Update meta with extra fields """
+    meta = load_meta(filename)
+    meta.update(extra)
+    write_meta(filename, meta)
+
+
 def _load_tag_class(gramtab_format, grammemes_filename):
     """ Load and initialize Tag class (according to ``gramtab_format``). """
     if gramtab_format not in tagset.registry:
