@@ -5,6 +5,7 @@ Analyzer units for unknown words with hyphens
 """
 
 from __future__ import absolute_import, unicode_literals, division
+from pymorphy2.dawg import PrefixMatcher
 
 from pymorphy2.units.base import BaseAnalyzerUnit, AnalogyAnalizerUnit
 from pymorphy2.units.utils import (add_parse_if_not_seen, add_tag_if_not_seen,
@@ -27,12 +28,12 @@ class HyphenSeparatedParticleAnalyzer(AnalogyAnalizerUnit):
         particles at tokenization level.
 
     """
-    ESTIMATE_DECAY = 0.9
+    def __init__(self, particles_after_hyphen, score_multiplier=0.9):
+        self.score_multiplier = score_multiplier
 
-    # XXX: maybe the code can be made faster by compiling this list to a DAWG?
-    PARTICLES_AFTER_HYPHEN = [
-        "-то", "-ка", "-таки", "-де", "-тко", "-тка", "-с", "-ста",
-    ]
+        # XXX: maybe the code can be made faster by compiling
+        # `particles_after_hyphen` list to a DAWG?
+        self.particles_after_hyphen = particles_after_hyphen
 
     def parse(self, word, word_lower, seen_parses):
 
@@ -45,7 +46,7 @@ class HyphenSeparatedParticleAnalyzer(AnalogyAnalizerUnit):
                     fixed_word+particle,
                     tag,
                     normal_form+particle,
-                    score*self.ESTIMATE_DECAY,
+                    score*self.score_multiplier,
                     methods_stack+(method,)
                 )
                 add_parse_if_not_seen(parse, result, seen_parses)
@@ -70,7 +71,7 @@ class HyphenSeparatedParticleAnalyzer(AnalogyAnalizerUnit):
         if '-' not in word:
             return
 
-        for particle in self.PARTICLES_AFTER_HYPHEN:
+        for particle in self.particles_after_hyphen:
             if not word.endswith(particle):
                 continue
 
@@ -97,10 +98,11 @@ class HyphenAdverbAnalyzer(BaseAnalyzerUnit):
 
     Example: по-западному
     """
-    ESTIMATE_DECAY = 0.7
+    def __init__(self, score_multiplier=0.7):
+        self.score_multiplier = score_multiplier
 
-    def __init__(self, morph):
-        super(HyphenAdverbAnalyzer, self).__init__(morph)
+    def init(self, morph):
+        super(HyphenAdverbAnalyzer, self).init(morph)
         self._tag = self.morph.TagClass('ADVB')
 
     def parse(self, word, word_lower, seen_parses):
@@ -109,7 +111,7 @@ class HyphenAdverbAnalyzer(BaseAnalyzerUnit):
 
         parse = (
             word_lower, self._tag, word_lower,
-            self.ESTIMATE_DECAY,
+            self.score_multiplier,
             ((self, word),)
         )
         seen_parses.add(parse)
@@ -146,8 +148,7 @@ class HyphenatedWordsAnalyzer(BaseAnalyzerUnit):
         * человек-гора -> человек + гора
 
     """
-    ESTIMATE_DECAY = 0.75
-
+    _repr_skip_value_params = ['skip_prefixes']
     _CONSIDER_THE_SAME = {
         'V-oy': 'V-ey',
         'gen1': 'gent',
@@ -156,11 +157,16 @@ class HyphenatedWordsAnalyzer(BaseAnalyzerUnit):
 
     }  # TODO: add more grammemes
 
-    def __init__(self, morph):
-        super(HyphenatedWordsAnalyzer, self).__init__(morph)
+    def __init__(self, skip_prefixes, score_multiplier=0.75):
+        self.score_multiplier = score_multiplier
+        self.skip_prefixes = skip_prefixes
+
+    def init(self, morph):
+        super(HyphenatedWordsAnalyzer, self).init(morph)
         Tag = morph.TagClass
         self._FEATURE_GRAMMEMES = (Tag.PARTS_OF_SPEECH | Tag.NUMBERS |
                                    Tag.CASES | Tag.PERSONS | Tag.TENSES)
+        self._has_skip_prefix = PrefixMatcher(self.skip_prefixes).is_prefixed
 
     def parse(self, word, word_lower, seen_parses):
         if not self._should_parse(word_lower):
@@ -182,7 +188,7 @@ class HyphenatedWordsAnalyzer(BaseAnalyzerUnit):
 
     def _parse_as_fixed_left(self, right_parses, seen, left):
         """
-        Step 1: Assume that the left part is an uninflected prefix.
+        Step 1: Assume that the left part is an immutable prefix.
         Examples: интернет-магазин, воздушно-капельный
         """
         result = []
@@ -198,7 +204,7 @@ class HyphenatedWordsAnalyzer(BaseAnalyzerUnit):
                 '-'.join((left, fixed_word)),
                 tag,
                 '-'.join((left, normal_form)),
-                score * self.ESTIMATE_DECAY,
+                score * self.score_multiplier,
                 new_methods_stack
             )
             result.append(parse)
@@ -242,7 +248,7 @@ class HyphenatedWordsAnalyzer(BaseAnalyzerUnit):
                     '-'.join((left_parse[0], right_parse[0])),  # word
                     left_tag,
                     '-'.join((left_parse[2], right_parse[2])),  # normal form
-                    left_parse[3] * self.ESTIMATE_DECAY,
+                    left_parse[3] * self.score_multiplier,
                     new_methods_stack
                 )
                 result.append(parse)
@@ -257,13 +263,6 @@ class HyphenatedWordsAnalyzer(BaseAnalyzerUnit):
             {'gen1': 'gent', 'loc1': 'loct'}
         )
 
-    def tag(self, word, word_lower, seen_tags):
-        result = []
-        # TODO: do not use self.parse
-        for p in self.parse(word, word_lower, set()):
-            add_tag_if_not_seen(p[1], result, seen_tags)
-        return result
-
     def _should_parse(self, word):
         if '-' not in word:
             return False
@@ -277,7 +276,7 @@ class HyphenatedWordsAnalyzer(BaseAnalyzerUnit):
             # require exactly 1 hyphen, in the middle of the word
             return False
 
-        if self.dict.prediction_prefixes.prefixes(word):
+        if self._has_skip_prefix(word):
             # such words should really be parsed by KnownPrefixAnalyzer
             return False
 
